from bge import logic
import math
import lutils as lut
from nlu import debug

class BinomialTreeNode:
	def __init__(self, game_object):
		super().__init__()

		self.gobj = game_object
		self.gobj['node'] = self

		self.left = None
		self.right = None

		self.set_lr_nodes()
		self.id = None

		if self.is_root():
			self.id = math.pow(self.furthest_child.length_to_root, 2)

			self.construct_children_ids()

	def set_lr_nodes(self):
		children = lut.list_tail(self.gobj.children, 2, None)
		self.left = None if not children[0] else BinomialTreeNode(children[0])
		self.right = None if not children[1] else BinomialTreeNode(children[1])

	def construct_children_ids(self):
		if self.left:
			self.left.id = self.next_id_left()
			self.left.construct_children_ids()

		if self.right:
			self.right.id = self.next_id_right()
			self.right.construct_children_ids()

	def next_id(self, direction):
		previous = 0 if not len(self.parents) else self.parents[0]['node'].id
		diff = math.fabs(self.id - previous)

		if direction == "right":
			return self.id + diff/2

		return self.id - diff/2

	def next_id_right(self):
		return self.next_id("right")

	def next_id_left(self):
		return self.next_id("left")


	# Properties

	@property
	def parents(self):
		if self.gobj.parent:
			return [self.gobj.parent] + \
					self.gobj.parent['node'].parents

		return []

	def is_root(self):
		return not self.length_to_root

	@property
	def length_to_root(self):
		return len(self.parents)

	@property
	def parent_recursive(self):
		return self.gobj if not len(self.parents) else self.parents[-1]

	@property
	def children(self):
		return [c for c in [self.left, self.right] if c]

	@property
	def children_recursive(self):

		if len(self.children):
			return self.children + \
				   self.left.children_recursive if self.left else [] + \
				   self.right.children_recursive if self.right else []

		return []

	@property
	def furthest_child(self):
		new_list = sorted(self.children_recursive, \
						  key=lambda k: k.length_to_root)

		return None if not len(new_list) else new_list[-1]


	#Debug

	def draw_lines(self):
		if self.left:
			to = self.left.gobj.worldPosition - self.gobj.worldPosition
			debug.draw_vec(self.gobj.worldPosition, to)
			self.left.draw_lines()

		if self.right:
			to = self.right.gobj.worldPosition - self.gobj.worldPosition
			debug.draw_vec(self.gobj.worldPosition, to)
			self.right.draw_lines()

	

def init (cont):
	BinomialTreeNode(cont.owner)

def main (cont):
	objects = logic.getCurrentScene().objects

	text_objs = [t for t in objects if "Text" in t.name]

	for t in text_objs:
		t['Text'] = str(objects[t['node']]['node'].id)

def draw (cont):
	first = logic.getCurrentScene().objects['node']
	first['node'].draw_lines()