from bge import logic
from mathutils import Vector
import lutils as lut
from nlu import debug


class Route:
	def __init__(self, start, end):

		self.start = start
		self.end = end

		self.starts = self.nodes_in_range(self.start, 3)
		self.ends = self.nodes_in_range(self.end, 3)

		#print(self.starts)

		self.roots = [i.parent_recursive['node'] for i in self.starts]

		self.shortest_route = self.find_shortest_route(self.roots, self.starts, self.ends)


	@property
	def path(self):
		return self.shortest_route

	def nodes_in_range(self, obj, range):
		position = obj.worldPosition

		nodes = [n['node'] for n in logic.getCurrentScene().objects if 'node' in n and \
			(n.worldPosition - position).magnitude <= range]

		return nodes

	def find_shortest_route(self, nrs, nss, nes):

		all_routes = [self.make_route(i, j, k) for i in nrs for j in nss for k in nes]
		sorted(all_routes, key=lambda k: len(k))

		return all_routes[-1]

	def make_route(self, nr, ns, ne):

		root_to_start = self.make_root_path(nr, ns)
		root_to_end = self.make_root_path(nr, ne)

		inter = lut.intersection(root_to_start, root_to_end)
		last_equal = inter[-1]

		only_start = lut.subtract(root_to_start, root_to_end)
		only_start.reverse()
		only_end = lut.subtract(root_to_end, root_to_start)


		route = only_start + [last_equal] + only_end

		return route


	def make_root_path(self, root, node):

		if not root:
			return []
		elif root.id > node.id:
			return [root] + \
				   self.make_root_path(root.left, node)
		elif root.id < node.id:
			return [root] + \
				   self.make_root_path(root.right, node)

		return [root]


def main():
	objects = logic.getCurrentScene().objects
	start = objects['start']
	end = objects['end']

	route = Route(start, end)
	node_path = route.path
	positions = [n.gobj.worldPosition + Vector ((0, 0, 0.1)) for n in node_path]

	debug.draw_vecs(positions, (255, 0, 0))
